<object data="http://www.sf.tv/videoplayer/embed/2a8583e0-91ac-4451-b3af-d63fbc0c0042"
        type="application/x-shockwave-flash"
        style="width:640px;height:386px">
    <param name="movie" value="http://www.sf.tv/videoplayer/embed/2a8583e0-91ac-4451-b3af-d63fbc0c0042"/>
    <param name="quality" value="high" />
    <param name="allowFullScreen" value="true" />
</object>
