{% load i18n %}

<script type="text/javascript" charset="utf-8">
    var FEINCMS_ITEM_EDITOR_GETTEXT = {
        'DELETE_MESSAGE': "{% trans 'Really delete item?' %}",
        'CHANGE_TEMPLATE': "{% trans 'Really change template? All changes are saved.' %}",
        'CHANGE_TEMPLATE_WITH_MOVE': "{% trans 'Really change template? All changes are saved and content from %(source_regions)s is moved to %(target_region)s.' %}",
        'MOVE_TO_REGION': "{% trans 'Move to region:' %}"
    };
</script>
