<object width="425" height="344">
<param name="movie" value="//www.youtube.com/v/{{ v }}?hl=en&fs=1&"></param>
<param name="allowFullScreen" value="true"></param><param name="allowscriptaccess" value="always"></param>
<embed src="//www.youtube.com/v/{{ v }}?hl=en&fs=1&" type="application/x-shockwave-flash"
    allowscriptaccess="always" allowfullscreen="true" width="425" height="344"></embed>
</object>
