{% load i18n %}
<div class="contactform">
<form method="post" action=".">
    {% csrf_token %}
    <table>
        {{ form }}
        <tr class="submit">
            <td>&nbsp;</td>
            <td><input type="submit" value="{% trans "Submit" %}" /></td>
        </tr>
    </table>
</form>
</div>
